package GII::Event::Command::ValInfo;

=head1 NAME

GII::Event::Command::ValInfo - Perl class for gii_cmddata_valinfo structures.

=head1 SYNOPSIS

    use GII::Event::Command::Valinfo;

    $ev = new GII::Event::Command::Valinfo;

    $ev->send($input);

    $number = $ev->number();
    $ev->number($number);

    $longname = $ev->longname();
    $ev->longname($longname);

    $shortname = $ev->shortname();
    $ev->shortname($shortname);

    $min = $ev->min();
    $ev->min($min);

    $center = $ev->center();
    $ev->center($center);

    $max = $ev->max();
    $ev->max($max);

    $phystype = $ev->phystype();
    $ev->phystype($phystype);

    $add = $ev->add();
    $ev->add($add);

    $mul = $ev->mul();
    $ev->mul($mul);

    $div = $ev->div();
    $ev->div($div);

    $shift = $ev->shift();
    $ev->shift($shift);

    ($add, $mul, $div, $shift) = $ev->si();
    $ev->si($add,$mul,$div,$shift);

    $si_value = $ev->si_value($value);
    $si_value = ($ev->add + $value) * $ev->mul / $ev->div * (2 ** $ev->shift);

=head1 DESCRIPTION

The GII::Event::Command::ValInfo subclass provides simple methods to
access the extra members of a LibGII gii_cmddata_valinfo structure
which cannot be accessed via the methods in the class GII::Event::Command.

=cut

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);
use GII::Event::Command;

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader GII::Event::Command);

=head1 Exported Constants

PT_UNKNOWN
PT_TIME
PT_FREQUENCY
PT_LENGTH
PT_VELOCITY
PT_ACCELERATION
PT_ANGLE
PT_ANGVELOCITY
PT_ANGACCELERATION
PT_AREA
PT_VOLUME
PT_MASS
PT_FORCE
PT_PRESSURE
PT_TORQUE
PT_ENERGY
PT_POWER
PT_TEMPERATURE
PT_CURRENT
PT_VOLTAGE
PT_RESISTANCE
PT_CAPACITY
PT_INDUCTIVITY	

=cut

@EXPORT = qw(
PT_UNKNOWN
PT_TIME
PT_FREQUENCY
PT_LENGTH
PT_VELOCITY
PT_ACCELERATION
PT_ANGLE
PT_ANGVELOCITY
PT_ANGACCELERATION
PT_AREA
PT_VOLUME
PT_MASS
PT_FORCE
PT_PRESSURE
PT_TORQUE
PT_ENERGY
PT_POWER
PT_TEMPERATURE
PT_CURRENT
PT_VOLTAGE
PT_RESISTANCE
PT_CAPACITY
PT_INDUCTIVITY	
);

$VERSION = '0.01';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
	if ($! =~ /Invalid/) {
	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
	    goto &AutoLoader::AUTOLOAD;
	}
	else {
		croak "Your vendor has not defined GII::Event::Command::ValInfo macro $constname";
	}
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

bootstrap GII::Event::Command::ValInfo $VERSION;

=head1 Methods

The extra methods and differences between GII::Event::Command methods
and the ones provided in the GII::Event::Command::ValInfo class are
as follows:

=head2 new

The GII::Event::Command::ValInfo constructor takes all the same arguments
as the GII::Event::Command constructor, and treats them the same with the
exception that it does not do any automagical subclassing and it has
a strictly fixed size.  All calls to this constructor will either fail
or allocate/reuse a gii_cmd_event/gii_cmddata_valinfo structure.

=cut

=head2 longname

With no arguments, gets a string containing the verbose name of the value.
With an argument, sets said string and returns the string set.  The
string is maximum 75 characters.

=cut

sub longname {
    my ($this) = shift();
    if (@_ + 0) {
        return set_longname($this->{obj}, shift());
    }
    return get_longname($this->{obj});
}

=head2 shortname

With no arguments, gets a string containing the compact name of the value.
With an argument, sets said string and returns the string set.  The
string is maximum 5 characters.

=cut

sub shortname {
    my ($this) = shift();
    if (@_ + 0) {
        return set_shortname($this->{obj}, shift());
    }
    return get_shortname($this->{obj});
}

=head2 min

With no arguments, gets the minumum value that the valuator can assume.
With an argument, sets said value and returns the value set.

=cut

sub min {
    my ($this) = shift();
    if (@_ + 0) {
        return set_min($this->{obj}, shift());
    }
    return get_min($this->{obj});
}

=head2 center

With no arguments, gets the normal center value that the valuator assumes.
With an argument, sets said value and returns the value set.

=cut

sub center {
    my ($this) = shift();
    if (@_ + 0) {
        return set_center($this->{obj}, shift());
    }
    return get_center($this->{obj});
}

=head2 max

With no arguments, gets the normal maximum value that the valuator can assume.
With an argument, sets said value and returns the value set.

=cut

sub max {
    my ($this) = shift();
    if (@_ + 0) {
        return set_max($this->{obj}, shift());
    }
    return get_max($this->{obj});
}

=head2 phystype

With no arguments, gets the type of physical SI units the value
represents.  The constants representing SI values can be imported by
explicitly using Event::Command::ValInfo, or can be used fully qualified from 
the Event::Command::ValInfo:: namespace.  With an argument, sets said 
value and returns the value set.

=cut

sub phystype {
    my ($this) = shift();
    if (@_ + 0) {
        return set_phystype($this->{obj}, shift());
    }
    return get_phystype($this->{obj});
}

=head2 add

With no arguments, gets the additive conversion offset for SI unit 
conversion for the value.  With an argument, sets said value and 
returns the value set.

=cut

sub add {
    my ($this) = shift();
    if (@_ + 0) {
        return set_add($this->{obj}, shift());
    }
    return get_add($this->{obj});
}

=head2 mul

With no arguments, gets the multiplicative conversion cooefficient for SI unit 
conversion for the value.  With an argument, sets said value and 
returns the value set.

=cut

sub mul {
    my ($this) = shift();
    if (@_ + 0) {
        return set_mul($this->{obj}, shift());
    }
    return get_mul($this->{obj});
}

=head2 div

With no arguments, gets the inverse multiplicative conversion cooeficient for 
SI unit conversion for the value.  With an argument, sets said value and 
returns the value set.

=cut

sub div {
    my ($this) = shift();
    if (@_ + 0) {
        return set_div($this->{obj}, shift());
    }
    return get_div($this->{obj});
}

=head2 shift

With no arguments, gets the power-of-two shift for SI unit conversion for 
the value.  With an argument, sets said value and returns the value set.

=cut

sub shift {
    my ($this) = shift();
    if (@_ + 0) {
        return set_shift($this->{obj}, shift());
    }
    return get_shift($this->{obj});
}

=head2 si

With no arguments, gets the conversion parameters for 
SI unit conversion for the value.  With an argument, sets said values and 
returns the values set.

=cut

sub si {
    my ($this) = shift();
    if (@_ + 0) {
        return set_add($this->{obj}, shift());
        return set_mul($this->{obj}, shift());
        return set_div($this->{obj}, shift());
        return set_shift($this->{obj}, shift());
    }
    return (get_add($this->{obj}),
	    get_mul($this->{obj}),
	    get_div($this->{obj}),
	    get_shift($this->{obj}));
}

=head2 si_value

Takes a single argument with a valuator value.  Returns the value 
converted into SI units.  Should return overloaded object class 
objects (e.g. Math::BigFloat) assuming overloaded class operations with 
normal perl scalars all return them as well.

=cut

sub si_value {
    my ($this) = shift();
    return  ($this->get_add($this->{obj}) + shift()) * 
	    $this->get_mul($this->{obj}) / $this->get_div($this->{obj})
	    * (2 ** $this->get_shift($this->{obj}));
}

=head2 size

Size always returns a fixed size, and when called with a parameter
will always set the event size to the same value.  There is no negotiation
of the size of a GII::Event::Command::ValInfo.  If you are recycling
an old event structure, calling the size method will any defined argument
will ensure the event size is set correctly.

=cut

# size method is inherited, C routines are overloaded.

=head2 data_size

data_size always returns a fixed size, and when called with a parameter
will always set the command data size to the same value.  There is no
negotiation of the size of a GII::Event::Command::ValInfo.  If you
are recycling an old event structure, calling the data_size method with
any defined argument will ensure that both the event size and the command
data size is set correctly.

=cut

# data_size method is inherited, C routines are overloaded.


1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event::Command(3pm), GII::Event::Valuator(3pm), GII(3pm), perl(1).

=cut
